using System;
using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;
using Microsoft.AspNetCore.Builder;
using Microsoft.AspNetCore.Hosting;
using Microsoft.AspNetCore.Http;
using Microsoft.Extensions.DependencyInjection;
using Microsoft.Extensions.Hosting;

namespace WebReporting
{
    public class Startup
    {
        // This method gets called by the runtime. Use this method to add services to the container.
        // For more information on how to configure your application, visit https://go.microsoft.com/fwlink/?LinkID=398940
        public void ConfigureServices(IServiceCollection services)
        {
        }

        // This method gets called by the runtime. Use this method to configure the HTTP request pipeline.
        public void Configure(IApplicationBuilder app, IWebHostEnvironment env)
        {
            if (env.IsDevelopment())
            {
                app.UseDeveloperExceptionPage();
            }

            app.UseRouting();

            app.UseEndpoints(endpoints =>
            {
                endpoints.MapGet("/cars/{make:regex(^(honda|mercedes|tesla)$)}/{year:range(2000,2020)}", async context =>
                {
                    try
                    {
                        var year = int.Parse(context.Request.RouteValues["year"].ToString());
                        var make = context.Request.RouteValues["make"].ToString();
                        var stream = GeneratePDF(make, year);
                        context.Response.ContentType = "application/pdf";
                        context.Response.Headers.Add("content-disposition", $"inline; filename={make}-{year}.pdf");
                        await stream.CopyToAsync(context.Response.Body);
                    }
                    catch(Exception ex)
                    {
                        await context.Response.WriteAsync(ex.ToString());
                    }

                });
                endpoints.MapGet("/", async context => 
                {
                    await context.Response.WriteAsync("Hello buddy!");
                });
            });
        }
        
        internal System.IO.Stream GeneratePDF(string make, int year)
        {
            using (var reportStream = typeof(Startup).Assembly.GetManifestResourceStream("WebReporting.Report.rdlx"))
            using (var reader = new System.IO.StreamReader(reportStream))
            {
                var rpt = new GrapeCity.ActiveReports.PageReport(reader);
                rpt.Document.Parameters["make"].CurrentValue = make;
                rpt.Document.Parameters["year"].CurrentValue = year;
                var pdfRe = new GrapeCity.ActiveReports.Export.Pdf.Page.PdfRenderingExtension();
                var output = new GrapeCity.ActiveReports.Rendering.IO.MemoryStreamProvider();
                rpt.Document.Render(pdfRe, output);
                return output.GetPrimaryStream().OpenStream();
            }
        }

    }
}
